{% load i18n %}
{% if perms.netbox_branching.archive_branch and branch.can_archive %}
  <a href="{% url 'plugins:netbox_branching:branch_archive' pk=branch.pk %}" class="btn btn-primary">
    <i class="mdi mdi-archive-outline"></i> {% trans "Archive" %}
  </a>
{% else %}
  <button type="button" class="btn btn-primary disabled" aria-disabled="true">
    <i class="mdi mdi-archive-outline"></i> {% trans "Archive" %}
  </button>
{% endif %}
